﻿namespace GenrerateSqlDescription
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Text.RegularExpressions;


    public class SqlNotesGenerator
    {
        private readonly Regex _tableReg = new Regex(@"/[\*]+([^\*]*)[\*]+/[\r\n\s]*CREATE TABLE \[dbo\].\[(\w*)\]");
        private readonly Regex _columnsReg = new Regex(@"\[([\w]*)\][^\/]*/[\*]+([^\*]*)[\*]+/");
        private const string TableDescriptionCrateSqlFormat = "EXEC sp_addextendedproperty 'MS_Description','{0}','user ',dbo,'table',[{1}];\r\n";
        private const string ColumnDescriptionCrateSqlFormat = "EXEC sp_addextendedproperty 'MS_Description','{0}','user',dbo,'table',[{1}],'column','{2}';\r\n";
        private const string TableDescriptionDropSqlFormat = "EXEC sp_dropextendedproperty 'MS_Description','user',dbo,'table',[{0}];\r\n";
        private const string ColumnescriptionDropSqlFormat = "EXEC sp_dropextendedproperty 'MS_Description','user',dbo,'table',[{0}],'column','{1}';\r\n";
        private const string CheckTableExistsSqlFormat = "IF OBJECT_ID(N'{0}',N'U') IS NOT NULL \r\nBEGIN \r\n";
        private const string EndStr = "END";
        private const string Tab = "	";

        public bool GenDrop { get; set; }
        public string Path { get; set; }

        private string GetDescriptionSql(string path)
        {
            var sb = new StringBuilder();
            var fs = File.OpenRead(path);
            var fileRd = new StreamReader(fs);
            var str = fileRd.ReadToEnd();
            var tableMatch = _tableReg.Match(str);
            if (tableMatch.Length < 2) return string.Empty;

            var tableName = tableMatch.Groups[2].Value;
            var tableDes = tableMatch.Groups[1].Value;
            if (string.IsNullOrEmpty(tableName) || string.IsNullOrEmpty(tableDes)) return string.Empty;

            var columnStr = str.Substring(str.IndexOf("(", StringComparison.Ordinal));
            var matches = _columnsReg.Matches(columnStr);

            sb.AppendFormat(CheckTableExistsSqlFormat, tableName);

            sb.Append(Tab);
            if (GenDrop)
                sb.AppendFormat(TableDescriptionDropSqlFormat, tableName);
            else
                sb.AppendFormat(TableDescriptionCrateSqlFormat, tableDes, tableName);
            foreach (Match match in matches)
            {
                var columnName = match.Groups[1].Value;
                var description = match.Groups[2].Value;
                if (string.IsNullOrEmpty(columnName) || string.IsNullOrEmpty(description))
                    continue;
                sb.Append(Tab);
                if (GenDrop)
                    sb.AppendFormat(ColumnescriptionDropSqlFormat, tableName, columnName);
                else
                    sb.AppendFormat(ColumnDescriptionCrateSqlFormat, description, tableName, columnName);
            }

            sb.AppendLine(EndStr);
            return sb.ToString();
        }


        public string GetGenerateSql()
        {
            Func<string, List<string>> getFolderSqlNotes = null;
            getFolderSqlNotes = path =>
            {
                var listAll = new List<string>();
                var files = Directory.GetFiles(path);
                var dirs = Directory.GetDirectories(path);
                foreach (string t in dirs)
                {
                    listAll.AddRange(getFolderSqlNotes(t)); ;
                }
                var listStr = files.Where(m => m.EndsWith(".sql")).Select(GetDescriptionSql).ToList();
                listAll.AddRange(listStr);
                return listAll;
            };
            var list = getFolderSqlNotes(Path);
            return string.Join("\r\n", list);
        }

    }

}
